@@ -1,7 +1,7 @@
 #
 # This file is part of Config-Model-Tester
 #
-# This software is Copyright (c) 2014 by Dominique Dumont.
+# This software is Copyright (c) 2015 by Dominique Dumont.
 #
 # This is free software, licensed under:
 #
@@ -9,7 +9,7 @@
 #
 package Config::Model::Tester;
 # ABSTRACT: Test framework for Config::Model
-$Config::Model::Tester::VERSION = '2.050';
+$Config::Model::Tester::VERSION = '2.051';
 use warnings;
 use strict;
 use locale;
@@ -257,6 +257,52 @@ sub check_annotation {
     }
 }
 
+sub has_key {
+    my ($root, $c, $nw) = @_;
+
+    _test_key($root, $c, $nw, 0);
+}
+
+sub has_not_key {
+    my ($root, $c, $nw) = @_;
+
+    _test_key($root, $c, $nw, 1);
+}
+
+sub _test_key {
+    my ($root, $c, $nw, $invert) = @_;
+
+    my @checks = ref $c eq 'ARRAY' ? @$c
+        : map { ( $_ => $c->{$_})} sort keys %$c ;
+
+    while (@checks) {
+        my $path       = shift @checks;
+        my $spec       = shift @checks;
+        my @key_checks = ref $spec eq 'ARRAY' ? @$spec: ($spec);
+
+        my $obj = $root->grab( step => $path, type => 'hash' );
+        my @keys = $obj->fetch_all_indexes;
+        my $res = 0;
+        foreach my $check (@key_checks) {
+            my @match  ;
+            foreach my $k (@keys) {
+                if (ref $check eq 'Regexp') {
+                    push @match, $k if $k =~ $check;
+                }
+                else {
+                    push @match, $k if $k eq $check;
+                }
+            }
+            if ($invert) {
+                is(scalar @match,0, "check $check matched no key" );
+            }
+            else {
+                ok(scalar @match, "check $check matched with keys @match" );
+            }
+        }
+    }
+}
+
 sub write_data_back {
     my ($model_test, $inst, $t) = @_;
     local $Config::Model::Value::nowarning = $t->{no_warnings} || 0;
@@ -401,6 +447,9 @@ sub run_model_test {
 
         check_data("first", $root, $t->{check}, $t->{no_warnings}) if $t->{check};
 
+        has_key     ( $root, $t->{has_key}, $t->{no_warnings}) if $t->{has_key} ;
+        has_not_key ( $root, $t->{has_not_key}, $t->{no_warnings}) if $t->{has_not_key} ;
+
         check_annotation($root,$t) if $t->{verify_annotation};
 
         write_data_back ($model_test, $inst, $t) ;
@@ -490,7 +539,7 @@ Config::Model::Tester - Test framework for Config::Model
 
 =head1 VERSION
 
-version 2.050
+version 2.051
 
 =head1 SYNOPSIS
 
@@ -765,6 +814,25 @@ A regexp can also be used to check value:
 
 =item *
 
+Verify if a hash contains one or more keys (or keys matching a regexp):
+
+ has_key => [
+    'sections' => 'debian', # sections must point to a hash element
+    'control' => [qw/source binary/],
+    'copyright Files' => qr/.c$/,
+    'copyright Files' => [qr/\.h$/], qr/\.c$/],
+ ],
+
+=item *
+
+Verify that a hash has B<not> a key (or a key matching a regexp):
+
+ has_not_key => [
+    'copyright Files' => qr/.virus$/ # silly, isn't ?
+ ],
+
+=item *
+
 Verify annotation extracted from the configuration file comments:
 
     verify_annotation => {
@@ -938,7 +1006,7 @@ Dominique Dumont
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is Copyright (c) 2014 by Dominique Dumont.
+This software is Copyright (c) 2015 by Dominique Dumont.
 
 This is free software, licensed under:
 
